import { generateFilterDescription } from "$lib/filter-description-generator";

export type ProjectTypeFilter = "Applications" | "Games" | "Projects";

export class Filter {
    projectType: ProjectTypeFilter = "Projects";

    languages: string[] = [];
    frameworks: string[] = [];
    technologies: string[] = [];
    genres: string[] = [];

    pageNo: number = 0;

    constructor(copyFilter?: Filter) {
        if (!copyFilter) {
            return;
        }

        this.projectType = copyFilter.projectType;
        this.languages = copyFilter.languages;
        this.frameworks = copyFilter.frameworks;
        this.technologies = copyFilter.technologies;
        this.genres = copyFilter.genres;
        this.pageNo = copyFilter.pageNo;
    }

    /*
     * Accessors
     */

    isApps = () => this.projectType === "Applications";
    isGames = () => this.projectType === "Games";
    isAll = () => this.projectType === "Projects";

    hasLang = (lang: string) => this.languages.includes(lang);
    hasFramework = (framework: string) => this.frameworks.includes(framework);
    hasTech = (tech: string) => this.technologies.includes(tech);
    hasGenre = (genre: string) => this.genres.includes(genre);

    /*
     * Mutators
     */

    apps(): Filter {
        const newFilter = new Filter(this);
        newFilter.genres = [];
        newFilter.projectType = "Applications";
        return newFilter;
    }

    games(): Filter {
        const newFilter = new Filter(this);
        newFilter.projectType = "Games";
        return newFilter;
    }

    projects(): Filter {
        const newFilter = new Filter(this);
        newFilter.projectType = "Projects";
        return newFilter;
    }

    addLang(lang: string): Filter {
        if (this.languages.includes(lang)) {
            return this;
        }

        const newFilter = new Filter(this);
        newFilter.languages = [lang, ...newFilter.languages];
        return newFilter;
    }

    removeLang(lang: string): Filter {
        const newFilter = new Filter(this);
        newFilter.languages = newFilter.languages.filter(x => x !== lang);
        return newFilter;
    }

    addFramework(framework: string): Filter {
        if (this.frameworks.includes(framework)) {
            return this;
        }

        const newFilter = new Filter(this);
        newFilter.frameworks = [framework, ...newFilter.frameworks];
        return newFilter;
    }

    removeFramework(framework: string): Filter {
        const newFilter = new Filter(this);
        newFilter.frameworks = newFilter.frameworks.filter(x => x !== framework);
        return newFilter;
    }

    addTechnology(tech: string): Filter {
        if (this.technologies.includes(tech)) {
            return this;
        }

        const newFilter = new Filter(this);
        newFilter.technologies = [tech, ...newFilter.technologies];
        return newFilter;
    }

    removeTechnology(tech: string): Filter {
        const newFilter = new Filter(this);
        newFilter.technologies = newFilter.technologies.filter(x => x !== tech);
        return newFilter;
    }

    addGenre(genre: string): Filter {
        if (this.genres.includes(genre)) {
            return this;
        }

        const newFilter = new Filter(this);
        newFilter.genres = [genre, ...newFilter.genres];
        return newFilter;
    }

    removeGenre(genre: string): Filter {
        const newFilter = new Filter(this);
        newFilter.genres = newFilter.genres.filter(x => x !== genre);
        return newFilter;
    }

    page(newPage: number): Filter {
        const newFilter = new Filter(this);
        newFilter.pageNo = newPage;
        return newFilter;
    }

    clear(): Filter {
        const newFilter = new Filter(this);
        newFilter.languages = [];
        newFilter.frameworks = [];
        newFilter.technologies = [];
        newFilter.genres = [];
        return newFilter;
    }

    /*
     * Misc
     */

    toSearchParams(): URLSearchParams {
        let searchParams = new URLSearchParams();

        // Project type
        switch (this.projectType) {
            case "Applications":
                searchParams.set("apps", "");
                break;
            case "Games":
                searchParams.set("games", "");
                break;
        }

        // Languages etc.
        if (this.languages.length > 0) {
            searchParams.set("languages", this.languages.join(','));
        }
        if (this.frameworks.length > 0) {
            searchParams.set("frameworks", this.frameworks.join(','));
        }
        if (this.technologies.length > 0) {
            searchParams.set("technologies", this.technologies.join(','));
        }
        if (this.genres.length > 0) {
            searchParams.set("genres", this.genres.join(','));
        }

        // Page number
        if (this.pageNo > 0) {
            searchParams.set("page", (this.pageNo + 1) + '');
        }

        return searchParams;
    }

    toQueryString(): string {
        let queryString = `?${this.toSearchParams().toString()}`;

        if (queryString.length === 1) {
            return "";
        }

        queryString = queryString.replace(/apps\=/g, 'apps');
        queryString = queryString.replace(/games\=/g, 'games');

        return queryString;
    }

    toString(): string {
        return generateFilterDescription(this.projectType, this.genres, this.languages, this.frameworks, this.technologies);
    }
}

export const searchParamsToFilter = (searchParams: URLSearchParams) => {
    let newFilter = new Filter();

    // Project type
    if (searchParams.has("apps")) {
        newFilter = newFilter.apps();
    } else if (searchParams.has("games")) {
        newFilter = newFilter.games();
    } else {
        newFilter = newFilter.projects();
    }

    // Languages etc.
    const languages = searchParams.get("languages");
    if (languages) {
        for (const lang of languages.split(',')) {
            newFilter = newFilter.addLang(lang);
        }
    }

    const frameworks = searchParams.get("frameworks");
    if (frameworks) {
        for (const framework of frameworks.split(',')) {
            newFilter = newFilter.addFramework(framework);
        }
    }

    const technologies = searchParams.get("technologies");
    if (technologies) {
        for (const tech of technologies.split(',')) {
            newFilter = newFilter.addTechnology(tech);
        }
    }

    const genres = searchParams.get("genres");
    if (genres) {
        for (const genre of genres.split(',')) {
            newFilter = newFilter.addGenre(genre);
        }
    }

    // Page number
    const pageNo = searchParams.get("page");
    if (pageNo) {
        newFilter = newFilter.page(Number.parseInt(pageNo) - 1);
    }

    return newFilter;
}
