import { parse } from 'yaml';
import dayjs from "dayjs";
import type { Filter } from './filter';
import { ProjectType, type MinimalProject, type Project, type Employment } from './types';

const PAGE_SIZE = 12;

export const markdownToEmployment = (frontMatter: any, content: any, slug: string): Employment => {
    // Parse date
    let date: string = dayjs(frontMatter.startDate).format("MMMM YYYY");
    if (frontMatter.duration) {
        date += ` for ${frontMatter.duration}`
    } else if (frontMatter.endDate) {
        const endDate = dayjs(frontMatter.endDate).format("MMMM YYYY");
        date += ` to ${endDate}`;
    } else {
        date += ' to now';
    }

    return {
        title: frontMatter.title,
        role: frontMatter.role,
        hours: frontMatter.hours,
        img: frontMatter.img ?? slug,
        date,
        _startDate: dayjs(frontMatter.startDate).unix(),

        content,
    };
}

export const markdownToProject = (frontMatter: any, content: any, slug: string): Project => {
    return {
        name: frontMatter.name,
        slug,
        dateOfCompletion: new Date(frontMatter.dateOfCompletion),

        summary: frontMatter.summary,
        content,
        type: (!frontMatter.genres) || (frontMatter.genres?.length === 0) ? ProjectType.APPLICATION : ProjectType.GAME,

        languages: frontMatter.languages ?? [],
        frameworks: frontMatter.frameworks ?? [],
        technologies: frontMatter.technologies ?? [],
        genres: frontMatter.genres ?? [],

        screenshots: frontMatter.screenshots,

        demoUrl: frontMatter.demoUrl ?? "",
        releaseUrl: frontMatter.releaseUrl ?? "",
        sourceUrl: frontMatter.sourceUrl ?? "",

        starred: frontMatter.starred ?? false,
    };
}

export const minifyProject = (project: Project): MinimalProject => ({
    name: project.name,
    slug: project.slug,

    summary: project.summary,
    type: project.type,

    languages: project.languages,
    frameworks: project.frameworks,
    technologies: project.technologies,
    genres: project.genres,

    starred: project.starred,
})

const filterProjectsByProjectType = (projects: Project[], filter: Filter) => {
    switch (filter.projectType) {
        case "Applications":
            projects = projects.filter((p: Project) => p.type === ProjectType.APPLICATION);
            break;
        case "Games":
            projects = projects.filter((p: Project) => p.type === ProjectType.GAME);
            break;
        default:
            projects = projects;
    }
    return projects;
};

const filterProjectsByTags = (projects: Project[], filter: Filter) => {
    // Base case: no filter applied
    if (filter.languages.length === 0 &&
        filter.frameworks.length === 0 &&
        filter.technologies.length === 0 &&
        filter.genres.length === 0) {
        return projects;
    }

    projects = projects.filter(p => {
        if (filter.languages.length > 0) {
            let languageCheck = false;
            for (const filterLang of filter.languages) {
                if (p.languages.includes(filterLang)) {
                    languageCheck = true;
                    break;
                }
            }
            if (!languageCheck) {
                return false;
            }
        }

        if (filter.frameworks.length > 0) {
            let frameworkCheck = false;
            for (const filterFramework of filter.frameworks) {
                if (p.frameworks.includes(filterFramework)) {
                    frameworkCheck = true;
                    break;
                }
            }
            if (!frameworkCheck) {
                return false;
            }
        }

        if (filter.technologies.length > 0) {
            let techCheck = false;
            for (const filterTech of filter.technologies) {
                if (p.technologies.includes(filterTech)) {
                    techCheck = true;
                    break;
                }
            }
            if (!techCheck) {
                return false;
            }
        }

        if (filter.genres.length > 0) {
            let genreCheck = false;
            for (const filterGenre of filter.genres) {
                if (p.genres.includes(filterGenre)) {
                    genreCheck = true;
                    break;
                }
            }
            if (!genreCheck) {
                return false;
            }
        }

        return true;
    });

    return projects;
};

const filterProjectsWithoutPagination = (projects: Project[], filter: Filter) => {
    projects = filterProjectsByProjectType(projects, filter);
    projects = filterProjectsByTags(projects, filter);
    return projects;
};

export const getPageCount = (projects: Project[], filter: Filter) => {
    projects = filterProjectsWithoutPagination(projects, filter);
    return Math.ceil(projects.length / PAGE_SIZE);
};

export const filterProjects = (projects: Project[], filter: Filter) => {
    projects = filterProjectsWithoutPagination(projects, filter);
    projects = projects.slice(filter.pageNo * PAGE_SIZE, filter.pageNo * PAGE_SIZE + PAGE_SIZE);
    return projects;
};

export const getSlugFromPath = (path: string) => {
    const slashSplit = path.split('/');
    const filename = slashSplit[slashSplit.length - 1];
    return filename.split('.')[0];
};
